\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : Ushi versie 3
\ DESCRIPTION : Simpele autonome en spelletjes robot
\ CATEGORY    : Applicatie, afmeting: 618 bytes.
\ AUTHOR      : Willem Ouwerkerk, February 07, 2001
\ LAST CHANGE : Willem Ouwerkerk, November 17, 2002
\ ----------------------------------------------------------------------

\ 90S2313                           \ Selecteer target
\ NEEDS TARGET.FRT                  \ Laad cpu specifiek deel
\ here

NEEDS RANDOM.FRT                    \ Gebruik random generator

PORTB SFR PB                        \ I/O-poorten v/d AT90S2313
PORTD SFR PD


\ Bit-i/o bezetting v/d AT90S2313   DDRx    ST(ekker)

PORTB 0 BIT-SFR VLOER-ACHTER        \ 0      9
PORTB 1 BIT-SFR VLOER-RECHTS        \ 0     12
PORTB 2 BIT-SFR RECHTS-OPZIJ        \ 0     13
PORTB 3 BIT-SFR RECHTS-VOOR         \ 0     14
PORTB 4 BIT-SFR ACHTER              \ 0     15
PORTB 5 BIT-SFR VLOER-LINKS         \ 0     17
PORTB 6 BIT-SFR LINKS-OPZIJ         \ 0     18
PORTB 7 BIT-SFR LINKS-VOOR          \ 0     19

PORTD 0 BIT-SFR GP2-DATA            \ 0      1      GP2 of USA ingang
PORTD 1 BIT-SFR GP2-CONTROL         \ 1      2      GP2 of USA uitgang
PORTD 2 BIT-SFR ZEND                \ 1      3      RC5-uitgang
PIND  3 BIT-SFR ONTVANG             \ 0      4      RC5-ingang
PORTD 4 BIT-SFR RESERVE             \ 1      8
PORTD 5 BIT-SFR MOTOR-LINKS         \ 1     10
PORTD 6 BIT-SFR MOTOR-RECHTS        \ 1     11


\ Ring- of tafelrand-detectie

\ Als er geen tafel meer is, dan is +n = 0, rand-achter is bit-0,
\ rand-linksvoor is bit-1, rand-rechtsvoor is bit-2
MACRO: TAFELRAND    ( -- +n )
    FROM VLOER-RECHTS 4 AND 
    FROM VLOER-LINKS  2 AND  OR
    FROM VLOER-ACHTER 1 AND  OR
    ;

\ True als de tafelrand is gesignaleerd !
MACRO: TAFELRAND?   ( -- vlag )
    TAFELRAND  0 <>
    ;

\ Als er geen ring is dan is +n = 0, ring-achter is bit-0,
\ ring-linksvoor is bit-1, rechtsvoor is bit-2
MACRO: VLOER         ( -- +n )
    FROM VLOER-RECHTS 0= 4 AND 
    FROM VLOER-LINKS 0=  2 AND  OR
    FROM VLOER-ACHTER 0= 1 AND  OR
    ;

\ True als een van de drie ringdetectors af gaat.
MACRO: VLOER?        ( -- vlag )
    VLOER  0 <>
    ;


\ Bumpers rondom

\ Als er geen bumper ingedrukt is is +n= 0
\ Bit-0 = rechts-voor, bit-1 = rechts opzij
\ Bit-2 = links-voor, bit-3 = links opzij
\ Bit-4 = bumper achterkant
: GESTOTEN          ( -- +n )
    FROM ACHTER 0=      16 AND
    FROM LINKS-OPZIJ 0=  8 AND  OR
    FROM LINKS-VOOR 0=   4 AND  OR
    FROM RECHTS-OPZIJ 0= 2 AND  OR
    FROM RECHTS-VOOR 0=  1 AND  OR
    ;


\ True als een van de vijf bumperschakelaars ingedrukt is.
MACRO: GESTOTEN?    ( -- vlag )
    GESTOTEN  0 <>
    ;


\ GP2D02 sensor om afstand te meten

MACRO: VOLGEND-BIT  ( x1 -- x2 )    \ Lees een databit
    SET GP2-CONTROL  1 /MS  CLEAR GP2-CONTROL
    1 /MS  2*  FROM GP2-DATA 0= 1 AND  OR
    ;

\ Geef afstand tot een object aan (lager is dichterbij).
\ Een meting duurt ongeveer 75 milliseconden.
: AFSTAND           ( -- afstand )  \ Lees afstand uit GP2D02 sensor
    SET GP2-DATA  CLEAR GP2-CONTROL \ activeer sensor en
    70 MS                           \ wacht even
    0  8 FOR                        \ begingetallen op de stack, neem
        VOLGEND-BIT                 \ acht maal een bit op
    NEXT
    SET GP2-CONTROL  2 MS           \ deactiveer sensor
    ;


\ Pulsbreedte regeling voor twee omgebouwde servomotoren

\ 0 = rechter motor  1 = linker motor
\ 832. 832.  4 CONSTANTS STILSTAND#   \ Puls +- 1,7 ms (oude HS300)
\ 764. 764.  4 CONSTANTS STILSTAND#   \ Puls +- 1,5 ms (nieuwe HS300)
  700. 700.  4 CONSTANTS STILSTAND#   \ Puls +- 1,4 ms (Carlson servo )

2 VARIABLES RICHTING                \ Onthoud draairichting
REGISTER TEMPO
REGISTER SERVO#

0 CONSTANT ||                       \ Bewegingsrichtingen
-1 CONSTANT ->
1 CONSTANT <-

0 CONSTANT RECHTER                  \ Motoraanduiding
1 CONSTANT LINKER

CODE SETUP-MOTOR    ( -- )          \ Zet hardware voor servo aansturing
    ADR SERVO# CLR,                 \ Begin bij servo-0
    R16 $80 LDI,                    \ Timer-1 interrupt aan
    TIMSK R16 OUT,
    R16 2 LDI,                      \ Prescaler= (deel cpu klok door) 8
    TCCR1B R16 OUT,
    R16 -10 LDI,                    \ Zet iets in het timer-1 register
    TCNT1H R16 OUT,
    TCNT1L R16 OUT,
( ) SEI,
    RET,
END-CODE


\ Zet de draairichting van de linker of rechter motor.
\ De pulsbreedte is regelbaar van 1.6 ms tot 1.8 ms,
\ dat is van vol achteruit tot vol vooruit.
MACRO: MOTOR             ( pulsbreedte motor# -- )
    TO RICHTING
    ;


\ Enkele timers voor Ushi

REGISTER TIK1                       \ Tel 20 ms tik
REGISTER TIK2                       \ Tel 100 ms tik
REGISTER TELLER1                    \ Tel 1/10 seconden
REGISTER TELLER2
VARIABLE TELLER3                    \ Tel seconden
VARIABLE TELLER4

CODE TIMERS         ( -- )          \ Verzorg de tijd voor Ushi
    ADR TIK1 DEC,
    ZER IF,
        R16 5 LDI,
        ADR TIK1 R16 MOV,
        ADR TELLER1 INC,            \ Tel van 0 tot 25,5 sec.
        ADR TELLER2 INC,
        ADR TIK2 DEC,
        ZER IF,                     \ Een sec. verstreken ?
            R16 10 LDI,
            ADR TIK2 R16 MOV,       \ Tel van 0 tot 255 sec.
            R16 ADR TELLER3 [Y] LD,
            R16 INC,
            ADR TELLER3 [Y] R16 ST,
            R16 ADR TELLER4 [Y] LD,
            R16 INC,
            ADR TELLER4 [Y] R16 ST,
        THEN,
    THEN,
    RET,
END-CODE

CODE SERVO-EXIT     ( -- )          \ 13 ticks
    TCNT1H R17 OUT,                 \ Zet pulslengte (time-out tijd)
    TCNT1L R16 OUT,
    ADR SERVO# INC,                 \ Naar volgende servo
    SREG R18 OUT,                   \ Herstel status register
    R18 POP,                        \ en gebruikte registers
    R17 POP,
    R16 POP,
    RETI,
END-CODE

\ De interrupt gebruikt 10.900 cycles is ~ 0.3 % cpu tijd.
\ Nu wordt runtime de pulsbreedte (lees timerwaarde) uitgerekend.
CODE SERVO-INT      ( -- )
    R16 PUSH,                       \ Bewaar te gebruiken registers
    R17 PUSH,
    R18 PUSH,
    R18 SREG IN,                    \ en het status register
    R16 ADR SERVO# MOV,             \ Welke servo is aan de beurt
    R16 0 CPI,
    ZER IF,                         \ Servo-0
        ADR MOTOR-RECHTS SBI,       \ Puls is hoog
        R17 CLR,
        R16 0 RICHTING LDS,         \ Haal draairichting op
        R16 TST,
        NZR IF,                     \ Niet gestopt ?
            R16 ADR TEMPO MOV,      \ Ja, haal snelheid op
            ZLE IF,                 \ Achteruit, draai teken om
                R16 NEG, R17 0 SBCI,
            THEN,
        THEN,
        R16 1 STILSTAND# SUBI,      \ Maak timer waarde
        R17 0 STILSTAND# SBCI,
        ' SERVO-EXIT GJMP,          \ 15+13=28
    THEN,
    R16 1 CPI,
    ZER IF,                         \ Servo-1
        ADR MOTOR-RECHTS CBI,
        ADR MOTOR-LINKS SBI,
        R17 CLR,
        R16 1 RICHTING LDS,         \ Haal draairichting op
        R16 TST,
        NZR IF,                     \ Niet gestopt ?
            R16 ADR TEMPO MOV,      \ Ja, haal snelheid op
            ZLE IF,                 \ Achteruit, draai teken om
                R16 NEG, R17 0 SBCI,
            THEN,
        THEN,
        R16 3 STILSTAND# SUBI,      \ Maak timer waarde
        R17 2 STILSTAND# SBCI,
        ' SERVO-EXIT GJMP,          \ 20+13=33
    THEN,
    ADR MOTOR-LINKS CBI,
    ' TIMERS GCALL,
    R16 SER,                        \ Begin weer bij het begin
    ADR SERVO# R16 MOV,
    R16 -8500. DROP LDI,
    R17 -8500. NIP LDI,
    ' SERVO-EXIT GJMP,              \ 32+13=45
END-CODE  T1-OVERFLOW


\ Snelheidsregeling en bewegingsrichting

: SNELHEID          ( snelheid -- )
    80 UMIN  TO TEMPO
    ;

\ Stop beide motoren door een puls van ongev. 1.7 ms. 
: STOP              ( -- )
    || RECHTER MOTOR  || LINKER MOTOR
    ;

: VOORUIT           ( -- )
    -> RECHTER MOTOR  -> LINKER MOTOR
    ;

: ACHTERUIT         ( -- )
    <- RECHTER MOTOR  <- LINKER MOTOR
    ;

: RECHTS            ( -- )
    || RECHTER MOTOR  -> LINKER MOTOR
    ;

: LINKS             ( -- )
    -> RECHTER MOTOR  || LINKER MOTOR
    ;

: RECHTSOM          ( -- )
    <- RECHTER MOTOR  -> LINKER MOTOR
    ;

: LINKSOM           ( -- )
    -> RECHTER MOTOR  <- LINKER MOTOR
    ;


\ Hier komt de RC5 zender/decoder (lichtgewicht-sumo & tikkertje)

INCLUDE RC5a.FRT
INCLUDE RC5ZEND.FRT


\ Hier komt de stappenteller voor twee motoren (doolhof)


\ Hier komt de zijgang detectie (doolhof)


\ Initialiseer de I/O poorten
MACRO: SETUP-USHI   ( -- )
    $00 SETDIR PB  $76 SETDIR PD    ( Zet richting van I/O )
    $FF TO PB  $FF TO PD            ( Activeer alle pullups )
    SETUP-MOTOR  SETUP-RANDOM  SETUP-RC
    5 TO TIK1  10 TO TIK2
    ;

\ here 2swap d- d.

                              ( Einde )
